@@ -5,6 +5,7 @@
 #include <signal.h>
 #include <time.h>
 #include <assert.h>
+#include <math.h>
 #ifndef WIN32
 #include <sys/types.h>
 #include <sys/wait.h>
@@ -350,7 +351,7 @@ void SMSD_Log_Function(const char *text, void *data)
 	newsize = pos + strlen(text) + 1;
 
 	/* Reallocate buffer if needed */
-	if (newsize > Config->gammu_log_buffer_size) {
+	if (newsize > Config->gammu_log_buffer_size || Config->gammu_log_buffer == NULL) {
 		newsize += 50;
 		Config->gammu_log_buffer = realloc(Config->gammu_log_buffer, newsize);
 		if (Config->gammu_log_buffer == NULL) {
@@ -804,6 +805,7 @@ GSM_Error SMSD_ReadConfig(const char *filename, GSM_SMSDConfig *Config, gboolean
 	Config->checksecurity = INI_GetBool(Config->smsdcfgfile, "smsd", "checksecurity", TRUE);
 	Config->hangupcalls = INI_GetBool(Config->smsdcfgfile, "smsd", "hangupcalls", FALSE);
 	Config->checksignal = INI_GetBool(Config->smsdcfgfile, "smsd", "checksignal", TRUE);
+	Config->checknetwork = INI_GetBool(Config->smsdcfgfile, "smsd", "checknetwork", TRUE);
 	Config->checkbattery = INI_GetBool(Config->smsdcfgfile, "smsd", "checkbattery", TRUE);
 	Config->enable_send = INI_GetBool(Config->smsdcfgfile, "smsd", "send", TRUE);
 	Config->enable_receive = INI_GetBool(Config->smsdcfgfile, "smsd", "receive", TRUE);
@@ -819,8 +821,8 @@ GSM_Error SMSD_ReadConfig(const char *filename, GSM_SMSDConfig *Config, gboolean
 
 	SMSD_Log(DEBUG_NOTICE, Config, "CommTimeout=%i, SendTimeout=%i, ReceiveFrequency=%i, ResetFrequency=%i, HardResetFrequency=%i",
 			Config->commtimeout, Config->sendtimeout, Config->receivefrequency, Config->resetfrequency, Config->hardresetfrequency);
-	SMSD_Log(DEBUG_NOTICE, Config, "checks: CheckSecurity=%d, CheckBattery=%d, CheckSignal=%d",
-			Config->checksecurity, Config->checkbattery, Config->checksignal);
+	SMSD_Log(DEBUG_NOTICE, Config, "checks: CheckSecurity=%d, CheckBattery=%d, CheckSignal=%d, CheckNetwork=%d",
+			Config->checksecurity, Config->checkbattery, Config->checksignal, Config->checknetwork);
 	SMSD_Log(DEBUG_NOTICE, Config, "mode: Send=%d, Receive=%d",
 			Config->enable_send, Config->enable_receive);
 
@@ -996,7 +998,7 @@ gboolean SMSD_CheckSecurity(GSM_SMSDConfig *Config)
 char *SMSD_RunOnCommand(const char *locations, const char *command)
 {
 	char *result;
-	int len;
+	size_t len;
 
 	assert(command != NULL);
 
@@ -1526,13 +1528,10 @@ gboolean SMSD_ReadDeleteSMS(GSM_SMSDConfig *Config)
 		for (j = 0; j < SortedSMS[i]->Number; j++) {
 			SortedSMS[i]->SMS[j].Folder = 0;
 			error = GSM_DeleteSMS(Config->gsm, &SortedSMS[i]->SMS[j]);
-			switch (error) {
-				case ERR_NONE:
-				case ERR_EMPTY:
-					break;
-				default:
-					SMSD_LogError(DEBUG_INFO, Config, "Error deleting SMS", error);
-					return FALSE;
+			// Empty error can happen if deleting message several times
+			if (error != ERR_NONE && error != ERR_EMPTY) {
+				SMSD_LogError(DEBUG_INFO, Config, "Error deleting SMS", error);
+				return FALSE;
 			}
 		}
 
@@ -1603,6 +1602,18 @@ void SMSD_PhoneStatus(GSM_SMSDConfig *Config) {
 	if (error != ERR_NONE) {
 		memset(&(Config->Status->Network), 0, sizeof(Config->Status->Network));
 	}
+	if (Config->checknetwork) {
+		error = GSM_GetNetworkInfo(Config->gsm, &Config->Status->NetInfo);
+	} else {
+		error = ERR_UNKNOWN;
+	}
+	if (error != ERR_NONE) {
+		memset(&(Config->Status->NetInfo), 0, sizeof(Config->Status->NetInfo));
+	} else if (error == ERR_NONE) {
+		if (Config->Status->NetInfo.State == GSM_NoNetwork) {
+			GSM_SetPower(Config->gsm, TRUE);
+		}
+	}
 }
 
 /**
@@ -1656,31 +1667,35 @@ GSM_Error SMSD_SendSMS(GSM_SMSDConfig *Config)
 	}
 
 	for (i = 0; i < sms.Number; i++) {
+		/* No SMSC set in message */
 		if (sms.SMS[i].SMSC.Location == 0 && UnicodeLength(sms.SMS[i].SMSC.Number) == 0 && Config->SMSC.Location == 0) {
 			SMSD_Log(DEBUG_INFO, Config, "Message without SMSC, using configured one");
-			memcpy(&sms.SMS[i].SMSC,&Config->SMSC,sizeof(GSM_SMSC));
-			sms.SMS[i].SMSC.Location = 0;
+			sms.SMS[i].SMSC = Config->SMSC;
 			if (Config->relativevalidity != -1) {
 				sms.SMS[i].SMSC.Validity.Format	  = SMS_Validity_RelativeFormat;
 				sms.SMS[i].SMSC.Validity.Relative = Config->relativevalidity;
 			}
 
 		}
+		/* Still nothing set after using configured one */
 		if (sms.SMS[i].SMSC.Location == 0 && UnicodeLength(sms.SMS[i].SMSC.Number) == 0) {
 			SMSD_Log(DEBUG_INFO, Config, "Message without SMSC, assuming you want to use the one from phone");
 			sms.SMS[i].SMSC.Location = 1;
 		}
+		/* Should use SMSC from phone */
 		if (sms.SMS[i].SMSC.Location != 0) {
+			/* Do we have cached entry? */
 			if (Config->SMSCCache.Location != sms.SMS[i].SMSC.Location) {
 				Config->SMSCCache.Location = sms.SMS[i].SMSC.Location;
-				error = GSM_GetSMSC(Config->gsm,&Config->SMSCCache);
+				error = GSM_GetSMSC(Config->gsm, &Config->SMSCCache);
 				if (error!=ERR_NONE) {
 					SMSD_Log(DEBUG_ERROR, Config, "Error getting SMSC from phone");
 					return ERR_UNKNOWN;
 				}
 
 			}
-			memcpy(&sms.SMS[i].SMSC,&Config->SMSCCache,sizeof(GSM_SMSC));
+			sms.SMS[i].SMSC = Config->SMSCCache;
+			/* Reset location to avoid reading from phone */
 			sms.SMS[i].SMSC.Location = 0;
 			if (Config->relativevalidity != -1) {
 				sms.SMS[i].SMSC.Validity.Format	  = SMS_Validity_RelativeFormat;
@@ -1733,6 +1748,7 @@ GSM_Error SMSD_SendSMS(GSM_SMSDConfig *Config)
 		Config->Status->Sent++;
 		error = Config->Service->AddSentSMSInfo(&sms, Config, Config->SMSID, i+1, SMSD_SEND_OK, Config->TPMR);
 		if (error != ERR_NONE) {
+			SMSD_LogError(DEBUG_INFO, Config, "Error setting sent status", error);
 			goto failure_sent;
 		}
 	}
@@ -1756,7 +1772,7 @@ failure_unsent:
 
 	Config->Service->UpdateRetries(Config, Config->SMSID);
 
-       sleep(60);
+	sleep(60);
 	return ERR_UNKNOWN;
 failure_sent:
 
@@ -1844,10 +1860,12 @@ GSM_Error SMSD_InitSharedMemory(GSM_SMSDConfig *Config, gboolean writable)
 			GetCompiler());
 		memset(&Config->Status->Charge, 0, sizeof(GSM_BatteryCharge));
 		memset(&Config->Status->Network, 0, sizeof(GSM_SignalQuality));
+		memset(&Config->Status->NetInfo, 0, sizeof(GSM_NetworkInfo));
 		Config->Status->Received = 0;
 		Config->Status->Failed = 0;
 		Config->Status->Sent = 0;
 		Config->Status->IMEI[0] = 0;
+		Config->Status->IMSI[0] = 0;
 	}
 	return ERR_NONE;
 }
@@ -1911,6 +1929,7 @@ GSM_Error SMSD_MainLoop(GSM_SMSDConfig *Config, gboolean exit_on_failure, int ma
 {
 	GSM_Error		error;
 	int                     errors = -1, initerrors=0;
+	unsigned int		lastsleep;
  	time_t			lastreceive = 0, lastreset = time(NULL), lasthardreset = time(NULL), lastnothingsent = 0, laststatus = 0;
 	time_t			lastloop = 0, current_time;
 	int i;
@@ -1989,7 +2008,7 @@ GSM_Error SMSD_MainLoop(GSM_SMSDConfig *Config, gboolean exit_on_failure, int ma
 				GSM_SetSendSMSStatusCallback(Config->gsm, SMSD_SendSMSStatusCallback, Config);
 				/* On first start we need to initialize some variables */
 				if (first_start) {
-					if (GSM_GetIMEI(Config->gsm, Config->Status->IMEI) != ERR_NONE) {
+					if (GSM_GetIMEI(Config->gsm, Config->Status->IMEI) != ERR_NONE || GSM_GetSIMIMSI(Config->gsm, Config->Status->IMSI) != ERR_NONE) {
 						errors++;
 					} else {
 						errors = 0;
@@ -2094,10 +2113,11 @@ GSM_Error SMSD_MainLoop(GSM_SMSDConfig *Config, gboolean exit_on_failure, int ma
 
 		/* Sleep some time before another loop */
 		current_time = time(NULL);
+		lastsleep = round(difftime(current_time, lastloop));
 		if (Config->loopsleep == 1) {
 			sleep(1);
-		} else if (difftime(current_time, lastloop) < Config->loopsleep) {
-			sleep(Config->loopsleep - difftime(current_time, lastloop));
+		} else if (lastsleep < Config->loopsleep) {
+			sleep(Config->loopsleep - lastsleep);
 		}
 	}
 	Config->Service->Free(Config);
