@@ -63,13 +63,13 @@ NORETURN void version(void)
 	printf("  - %s\n", "ODBC");
 #endif
 	printf("\n");
-	printf("Copyright (C) 2003 - 2015 Michal Cihar <michal@cihar.com> and other authors.\n");
+	printf("Copyright (C) 2003 - 2016 Michal Cihar <michal@cihar.com> and other authors.\n");
 	printf("\n");
 	printf("License GPLv2: GNU GPL version 2 <https://spdx.org/licenses/GPL-2.0>.\n");
 	printf("This is free software: you are free to change and redistribute it.\n");
 	printf("There is NO WARRANTY, to the extent permitted by law.\n");
 	printf("\n");
-	printf("Check <http://wammu.eu/gammu/> for updates.\n");
+	printf("Check <https://wammu.eu/gammu/> for updates.\n");
 	printf("\n");
 	exit(0);
 }
@@ -171,6 +171,7 @@ void process_commandline(int argc, char **argv, SMSD_Parameters * params)
 				help();
 				exit(0);
 			default:
+				fprintf(stderr, "Parameter -%c not known!\n", opt);
 				wrong_params();
 				break;
 		}
@@ -255,10 +256,11 @@ int main(int argc, char **argv)
 			return 3;
 		}
 		if (compact) {
-			printf("%s;%s;%s;%d;%d;%d;%d;%d\n",
+			printf("%s;%s;%s;%s;%d;%d;%d;%d;%d\n",
 				 status.Client,
 				 status.PhoneID,
 				 status.IMEI,
+				 status.IMSI,
 				 status.Sent,
 				 status.Received,
 				 status.Failed,
@@ -268,6 +270,7 @@ int main(int argc, char **argv)
 			printf("Client: %s\n", status.Client);
 			printf("PhoneID: %s\n", status.PhoneID);
 			printf("IMEI: %s\n", status.IMEI);
+			printf("IMSI: %s\n", status.IMSI);
 			printf("Sent: %d\n", status.Sent);
 			printf("Received: %d\n", status.Received);
 			printf("Failed: %d\n", status.Failed);
