@@ -1,9 +1,9 @@
 /**
- * libsql database service
+ * SQL database service
  *
  * Part of Gammu project
  *
- * Copyright (C) 2009 - 2015 Michal Čihař
+ * Copyright (C) 2009 - 2016 Michal Čihař
  *           (c) 2010 Miloslav Semler
  *
  * Licensed under GNU GPL version 2 or later
@@ -49,6 +49,7 @@ const char now_plus_pgsql[] = "now() + interval '%d seconds'";
 const char now_plus_sqlite[] = "datetime('now', '+%d seconds', 'localtime')";
 const char now_plus_freetds[] = "DATEADD('second', %d, CURRENT_TIMESTAMP)";
 const char now_plus_access[] = "now()+#00:00:%d#";
+const char now_plus_oracle[] = "CURRENT_TIMESTAMP + INTERVAL '%d' SECOND'";
 const char now_plus_fallback[] = "NOW() + INTERVAL %d SECOND";
 
 
@@ -69,6 +70,8 @@ static const char *SMSDSQL_NowPlus(GSM_SMSDConfig * Config, int seconds)
 		sprintf(result, now_plus_freetds, seconds);
 	} else if (strcasecmp(driver_name, "access") == 0) {
 		sprintf(result, now_plus_access, seconds);
+	} else if (strcasecmp(driver_name, "oracle") == 0) {
+		sprintf(result, now_plus_oracle, seconds);
 	} else if (strcasecmp(driver_name, "odbc") == 0) {
 		sprintf(result, now_plus_odbc, seconds);
 	} else {
@@ -81,7 +84,7 @@ const char escape_char_odbc[] = "";
 const char escape_char_mysql[] = "`";
 const char escape_char_pgsql[] = "\"";
 const char escape_char_sqlite[] = "";
-const char escape_char_freetds[] = "";
+const char escape_char_freetds[] = "\"";
 const char escape_char_fallback[] = "";
 
 static const char *SMSDSQL_EscapeChar(GSM_SMSDConfig * Config)
@@ -96,7 +99,7 @@ static const char *SMSDSQL_EscapeChar(GSM_SMSDConfig * Config)
 		return escape_char_pgsql;
 	} else if (strncasecmp(driver_name, "sqlite", 6) == 0) {
 		return escape_char_sqlite;
-	} else if (strcasecmp(driver_name, "freetds") == 0 || strcasecmp(driver_name, "mssql") == 0 || strcasecmp(driver_name, "sybase") == 0) {
+	} else if (strcasecmp(driver_name, "oracle") == 0 || strcasecmp(driver_name, "freetds") == 0 || strcasecmp(driver_name, "mssql") == 0 || strcasecmp(driver_name, "sybase") == 0) {
 		return escape_char_freetds;
 	} else if (strcasecmp(Config->driver, "odbc") == 0) {
 		return escape_char_odbc;
@@ -105,6 +108,32 @@ static const char *SMSDSQL_EscapeChar(GSM_SMSDConfig * Config)
 	}
 }
 
+const char rownum_clause_fallback[] = "";
+
+
+static const char *SMSDSQL_RownumClause(GSM_SMSDConfig * Config, const char *count, gboolean in_where)
+{
+	const char *driver_name;
+	static char result[100];
+
+	driver_name = SMSDSQL_SQLName(Config);
+
+	if (strcasecmp(driver_name, "oracle") == 0 || strcasecmp(driver_name, "freetds") == 0) {
+		if (in_where) {
+			strcpy(result, " AND ");
+		} else {
+			strcpy(result, " WHERE ");
+		}
+		strcat(result, "ROWNUM <= ");
+		strcat(result, count);
+		strcat(result, " ");
+		return result;
+	} else {
+		return rownum_clause_fallback;
+	}
+}
+
+
 const char top_clause_access[] = "TOP";
 const char top_clause_fallback[] = "";
 
@@ -136,7 +165,7 @@ static const char *SMSDSQL_LimitClause(GSM_SMSDConfig * Config, const char *coun
 
 	driver_name = SMSDSQL_SQLName(Config);
 
-	if (strcasecmp(driver_name, "access") == 0) {
+	if (strcasecmp(driver_name, "access") == 0 || strcasecmp(driver_name, "freetds") == 0 || strcasecmp(driver_name, "oracle") == 0) {
 		return limit_clause_access;
 	} else {
 		strcpy(result, limit_clause_fallback);
@@ -173,7 +202,7 @@ static const char *SMSDSQL_CurrentTime(GSM_SMSDConfig * Config)
 		return currtime_pgsql;
 	} else if (strncasecmp(driver_name, "sqlite", 6) == 0) {
 		return currtime_sqlite;
-	} else if (strcasecmp(driver_name, "freetds") == 0 || strcasecmp(driver_name, "mssql") == 0 || strcasecmp(driver_name, "sybase") == 0) {
+	} else if (strcasecmp(Config->driver, "oracle") == 0 || strcasecmp(driver_name, "freetds") == 0 || strcasecmp(driver_name, "mssql") == 0 || strcasecmp(driver_name, "sybase") == 0) {
 		return currtime_freetds;
 	} else if (strcasecmp(Config->driver, "odbc") == 0) {
 		return currtime_odbc;
@@ -193,7 +222,7 @@ static const char *SMSDSQL_Now(GSM_SMSDConfig * Config)
 		return now_pgsql;
 	} else if (strncasecmp(driver_name, "sqlite", 6) == 0) {
 		return now_sqlite;
-	} else if (strcasecmp(driver_name, "freetds") == 0 || strcasecmp(driver_name, "mssql") == 0 || strcasecmp(driver_name, "sybase") == 0) {
+	} else if (strcasecmp(Config->driver, "oracle") == 0 || strcasecmp(driver_name, "freetds") == 0 || strcasecmp(driver_name, "mssql") == 0 || strcasecmp(driver_name, "sybase") == 0) {
 		return now_freetds;
 	} else if (strcasecmp(Config->driver, "access") == 0) {
 		return now_access;
@@ -289,24 +318,6 @@ static GSM_Error SMSDSQL_NamedQuery(GSM_SMSDConfig * Config, const char *sql_que
 	int n, argc = 0;
 	struct GSM_SMSDdbobj *db = Config->db;
 
-	GSM_NetworkInfo NetInfo;
-	char empty[1] = "";
-	char *NetCode, *NetName;
-
-	NetCode = empty;
-	NetName = empty;
-
-	/* Query network status only if we need it */
-	if (
-		(strstr(sql_query, "%O") != NULL || strstr(sql_query, "%M") != NULL)
-		&& GSM_GetNetworkInfo(Config->gsm, &NetInfo) == ERR_NONE
-	) {
-		NetCode = NetInfo.NetworkCode;
-		if (NetInfo.NetworkName[0] != 0x00 || NetInfo.NetworkName[1] != 0x00) {
-			NetName = DecodeUnicodeConsole(NetInfo.NetworkName);
-		}
-	}
-
 	if (params != NULL) {
 		while (params[argc].type != SQL_TYPE_NONE) argc++;
 	}
@@ -324,7 +335,7 @@ static GSM_Error SMSDSQL_NamedQuery(GSM_SMSDConfig * Config, const char *sql_que
 			if (n < argc && n >= 0) {
 				switch(params[n].type){
 					case SQL_TYPE_INT:
-						ptr += sprintf(ptr, "%i", params[n].v.i);
+						ptr += sprintf(ptr, "%lli", params[n].v.i);
 						break;
 					case SQL_TYPE_STRING:
 						buffer2 = db->QuoteString(Config, params[n].v.s);
@@ -350,14 +361,17 @@ static GSM_Error SMSDSQL_NamedQuery(GSM_SMSDConfig * Config, const char *sql_que
 			case 'I':
 				to_print = Config->Status->IMEI;
 				break;
+			case 'S':
+				to_print = Config->Status->IMSI;
+				break;
 			case 'P':
 				to_print = Config->PhoneID;
 				break;
 			case 'O':
-				to_print = NetCode;
+				to_print = Config->Status->NetInfo.NetworkCode;
 				break;
 			case 'M':
-				to_print = NetName;
+				to_print = DecodeUnicodeConsole(Config->Status->NetInfo.NetworkName);
 				break;
 			case 'N':
 				snprintf(static_buff, sizeof(static_buff), "Gammu %s, %s, %s", GAMMU_VERSION, GetOS(), GetCompiler());
@@ -478,7 +492,14 @@ static GSM_Error SMSDSQL_CheckTable(GSM_SMSDConfig * Config, const char *table)
 	GSM_Error error;
 	struct GSM_SMSDdbobj *db = Config->db;
 
-	sprintf(buffer, "SELECT %s * FROM %s %s", SMSDSQL_TopClause(Config, "1"), table, SMSDSQL_LimitClause(Config, "1"));
+	sprintf(
+		buffer,
+		"SELECT %s * FROM %s %s %s",
+		SMSDSQL_TopClause(Config, "1"),
+		table,
+		SMSDSQL_RownumClause(Config, "1", FALSE),
+		SMSDSQL_LimitClause(Config, "1")
+	);
 	error = SMSDSQL_Query(Config, buffer, &res);
 	if (error != ERR_NONE) {
 		SMSD_Log(DEBUG_ERROR, Config, "Table %s not found, disconnecting!", table);
@@ -525,12 +546,6 @@ static GSM_Error SMSDSQL_Init(GSM_SMSDConfig * Config)
 	if (error != ERR_NONE)
 		return error;
 
-	error = SMSDSQL_CheckTable(Config, Config->table_daemons);
-	if (error != ERR_NONE) {
-		SMSD_Log(DEBUG_ERROR, Config, "Failed to open table %s", Config->table_daemons);
-		return error;
-	}
-
 	error = SMSDSQL_CheckTable(Config, Config->table_gammu);
 	if (error != ERR_NONE) {
 		SMSD_Log(DEBUG_ERROR, Config, "Failed to open table %s", Config->table_gammu);
@@ -581,7 +596,7 @@ static GSM_Error SMSDSQL_Init(GSM_SMSDConfig * Config)
 		db->Free(Config);
 		return ERR_UNKNOWN;
 	}
-	version = db->GetNumber(Config, &res, 0);
+	version = (int)db->GetNumber(Config, &res, 0);
 	db->FreeResult(Config, &res);
 	error = SMSD_CheckDBVersion(Config, version);
 	if (error != ERR_NONE) {
@@ -876,8 +891,8 @@ static GSM_Error SMSDSQL_FindOutboxSMS(GSM_MultiSMSMessage * sms, GSM_SMSDConfig
 	sms->Number = 0;
 	for (i = 0; i < GSM_MAX_MULTI_SMS; i++) {
 		GSM_SetDefaultSMSData(&sms->SMS[i]);
-		sms->SMS[i].SMSC.Number[0] = 0;
-		sms->SMS[i].SMSC.Number[1] = 0;
+		/* Force using default SMSC */
+		sms->SMS[i].SMSC.Location = 0;
 	}
 
 	for (i = 1; i < GSM_MAX_MULTI_SMS + 1; i++) {
@@ -981,16 +996,16 @@ static GSM_Error SMSDSQL_FindOutboxSMS(GSM_MultiSMSMessage * sms, GSM_SMSDConfig
 			}
 		}
 
-		sms->SMS[sms->Number].Class = db->GetNumber(Config, &res, 3);
+		sms->SMS[sms->Number].Class = (int)db->GetNumber(Config, &res, 3);
 		sms->SMS[sms->Number].PDU = SMS_Submit;
 		sms->Number++;
 
 		if (i == 1) {
 			strcpy(Config->CreatorID, db->GetString(Config, &res, 10));
-			Config->relativevalidity = db->GetNumber(Config, &res, 8);
+			Config->relativevalidity = (int)db->GetNumber(Config, &res, 8);
 
 			Config->currdeliveryreport = db->GetBool(Config, &res, 9);
-			Config->retries = db->GetNumber(Config, &res, 11);
+			Config->retries = (int)db->GetNumber(Config, &res, 11);
 
 			/* Is this a multipart message? */
 			if (!db->GetBool(Config, &res, 7)) {
@@ -1039,7 +1054,7 @@ static GSM_Error SMSDSQL_CreateOutboxSMS(GSM_MultiSMSMessage * sms, GSM_SMSDConf
 {
 	char creator[200];
 	int i;
-	unsigned int ID = 0;
+	unsigned long long ID = 0;
 	SQL_result res;
 	SQL_Var vars[6];
 	struct GSM_SMSDdbobj *db = Config->db;
@@ -1083,9 +1098,9 @@ static GSM_Error SMSDSQL_CreateOutboxSMS(GSM_MultiSMSMessage * sms, GSM_SMSDConf
 		}
 		db->FreeResult(Config, &res);
 	}
-	SMSD_Log(DEBUG_INFO, Config, "Written message with ID %u", ID);
+	SMSD_Log(DEBUG_INFO, Config, "Written message with ID %llu", ID);
 	if (NewID != NULL)
-		sprintf(NewID, "%d", ID);
+		sprintf(NewID, "%llu", ID);
 	return ERR_NONE;
 }
 
@@ -1262,10 +1277,6 @@ GSM_Error SMSDSQL_ReadConfiguration(GSM_SMSDConfig *Config)
 		Config->database="sms";
 	}
 
-	Config->table_daemons = INI_GetValue(Config->smsdcfgfile, "tables", "daemons", FALSE);
-	if (Config->table_daemons == NULL){
-		Config->table_daemons = "daemons";
-	}
 	Config->table_gammu = INI_GetValue(Config->smsdcfgfile, "tables", "gammu", FALSE);
 	if (Config->table_gammu == NULL){
 		Config->table_gammu = "gammu";
@@ -1303,24 +1314,33 @@ GSM_Error SMSDSQL_ReadConfiguration(GSM_SMSDConfig *Config)
 	}
 
 	Config->db = NULL;
-#ifdef HAVE_MYSQL_MYSQL_H
 	if (!strcasecmp(Config->driver, "native_mysql")) {
+#ifdef HAVE_MYSQL_MYSQL_H
 		Config->db = &SMSDMySQL;
-	}
+#else
+		SMSD_Log(DEBUG_ERROR, Config, "The %s driver was not compiled in!", Config->driver);
+		return ERR_DISABLED;
 #endif
-#ifdef HAVE_POSTGRESQL_LIBPQ_FE_H
+	}
 	if (!strcasecmp(Config->driver, "native_pgsql")) {
+#ifdef HAVE_POSTGRESQL_LIBPQ_FE_H
 		Config->db = &SMSDPgSQL;
-	}
+#else
+		SMSD_Log(DEBUG_ERROR, Config, "The %s driver was not compiled in!", Config->driver);
+		return ERR_DISABLED;
 #endif
-#ifdef ODBC_FOUND
+	}
 	if (!strcasecmp(Config->driver, "odbc")) {
+#ifdef ODBC_FOUND
 		Config->db = &SMSDODBC;
 		if (Config->sql == NULL) {
 			SMSD_Log(DEBUG_INFO, Config, "Using generic SQL for ODBC, this might fail. In such case please set SQL configuration option.");
 		}
-	}
+#else
+		SMSD_Log(DEBUG_ERROR, Config, "The %s driver was not compiled in!", Config->driver);
+		return ERR_DISABLED;
 #endif
+	}
 	if (Config->db == NULL) {
 #ifdef LIBDBI_FOUND
 		Config->db = &SMSDDBI;
@@ -1344,6 +1364,7 @@ GSM_Error SMSDSQL_ReadConfiguration(GSM_SMSDConfig *Config)
 	if (SMSDSQL_option(Config, SQL_QUERY_INSERT_PHONE, "insert_phone",
 		"INSERT INTO ", Config->table_phones, " (",
 			ESCAPE_FIELD("IMEI"),
+			", ", ESCAPE_FIELD("IMSI"),
 			", ", ESCAPE_FIELD("ID"),
 			", ", ESCAPE_FIELD("NetCode"),
 			", ", ESCAPE_FIELD("NetName"),
@@ -1354,7 +1375,7 @@ GSM_Error SMSDSQL_ReadConfiguration(GSM_SMSDConfig *Config)
 			", ", ESCAPE_FIELD("Client"),
 			", ", ESCAPE_FIELD("Battery"),
 			", ", ESCAPE_FIELD("Signal"),
-			") VALUES (%I, %P, %O, %M, %1, %2, ",
+			") VALUES (%I, %S, %P, %O, %M, %1, %2, ",
 			SMSDSQL_Now(Config),
 			", ",
 			SMSDSQL_NowPlus(Config, 10),
@@ -1446,8 +1467,9 @@ GSM_Error SMSDSQL_ReadConfiguration(GSM_SMSDConfig *Config)
 			" AND ", ESCAPE_FIELD("SendingTimeOut"), " < ", SMSDSQL_Now(Config),
 			" AND ", ESCAPE_FIELD("SendBefore"), " >= ", SMSDSQL_CurrentTime(Config),
 			" AND ", ESCAPE_FIELD("SendAfter"), " <= ", SMSDSQL_CurrentTime(Config),
-			" AND ( ", ESCAPE_FIELD("SenderID"), " is NULL OR ", ESCAPE_FIELD("SenderID"), " = '' OR ", ESCAPE_FIELD("SenderID"), " = %P )"
-			" ORDER BY ", ESCAPE_FIELD("InsertIntoDB"), " ASC ", SMSDSQL_LimitClause(Config, "%1"), NULL) != ERR_NONE) {
+			" AND ( ", ESCAPE_FIELD("SenderID"), " is NULL OR ", ESCAPE_FIELD("SenderID"), " = '' OR ", ESCAPE_FIELD("SenderID"), " = %P )",
+			SMSDSQL_RownumClause(Config, "%1", TRUE),
+			" ORDER BY ", ESCAPE_FIELD("Priority"), " DESC, ", ESCAPE_FIELD("InsertIntoDB"), " ASC ", SMSDSQL_LimitClause(Config, "%1"), NULL) != ERR_NONE) {
 		return ERR_UNKNOWN;
 	}
 
